@@ -122,7 +122,7 @@ shape <- function(...) {
         return(map_int(as.list(as_r_value(x$as_list())),
                        function(e) e %||% NA_integer_))
 
-      shp <- keras$ops$shape(x)
+      shp <- ops$shape(x)
 
       # convert subclassed tuples, as encountered in Torch
       # class(shp): torch.Size, python.builtin.tuple, python.builtin.object
@@ -202,6 +202,16 @@ as.integer.keras_shape <- function(x, ...) {
   vapply(x, function(el) el %||% NA_integer_, 1L)
 }
 
+#' @rdname shape
+#' @export
+#' @param na.rm passed on to Summary group generics like `prod()`. Unknown axes
+#'   are treated as `NA`.
+Summary.keras_shape <- function(..., na.rm = FALSE) {
+  x <- shape(...)
+  x <- as.integer.keras_shape(x)
+  do.call(.Generic, list(x, na.rm = na.rm))
+}
+
 #' @importFrom zeallot destructure
 #' @export
 destructure.keras_shape <- function(x) unclass(x)
@@ -229,4 +239,5 @@ as.list.keras_shape <- function(x, ...) unclass(x)
 
 # ' @rdname shape
 # ' @export
+# ## just use shape(shp1, shp2, ...)
 # c.keras_shape <- function(...) shape(...)
